﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Linq;
using System.Data.Entity;
using Example.DataAccess;

namespace Example.DataAccess.UnitTests
{
    [TestClass]
    public class BasicTests
    {
        [TestMethod]
        public void UpdateSingleEntity()        
        {
            Primary existing = null;
            String existingValue = String.Empty;

            using (DataAccessor a = new DataAccessor())
            {
                existing = a.DataContext.Primaries.Include("Secondaries").First();
                Assert.IsNotNull(existing);
                existingValue = existing.Title;
                existing.Title = "Unit " + DateTime.Now.ToString("MMdd hh:mm:ss");
            }
            using (DataAccessor b = new DataAccessor())
            {
                existing.State = ObjectState.Modified;
                b.InsertOrUpdate<Primary>(existing);
            }
            using (DataAccessor c = new DataAccessor())
            {
                existing.Title = existingValue;
                existing.State = ObjectState.Modified;
                c.InsertOrUpdate<Primary>(existing);
            }
        }

        [TestMethod]
        public void UpdateSingleEntitySingleContext()
        {
            Primary existing = null;
            String existingValue = String.Empty;

            using (DataAccessor a = new DataAccessor())
            {
                existing = a.DataContext.Primaries.Include("Secondaries").First();
                Assert.IsNotNull(existing);
                existingValue = existing.Title;
                existing.Title = "Unit " + DateTime.Now.ToString("MMdd hh:mm:ss");
           
                existing.State = ObjectState.Modified;
                a.InsertOrUpdate<Primary>(existing);
           
                existing.Title = existingValue;
                existing.State = ObjectState.Modified;
                a.InsertOrUpdate<Primary>(existing);
            }
        }

        [TestMethod]
        public void UpdateManyToMany()
        {
            Primary existing = null;
            Secondary other = null;
            String existingValue = String.Empty;
            String existingOtherValue = String.Empty;

            using (DataAccessor a = new DataAccessor())
            {
                //Note that we include the navigation property in the query
                existing = a.DataContext.Primaries.Include("Secondaries").First();
                Assert.IsTrue(existing.Secondaries.Count() > 1, "Should be at least 1 linked item");
            }
            //save the original description
            existingValue = existing.Description;
            //set a new dummy value (with a date/time so we can see it working)
            existing.Description = "Edit " + DateTime.Now.ToString("yyyyMMdd hh:mm:ss");
            existing.State = ObjectState.Modified;
                                
            other = existing.Secondaries.First();
            //save the original value
            existingOtherValue = other.AlternateDescription;
            //set a new value
            other.AlternateDescription = "Edit " + DateTime.Now.ToString("yyyyMMdd hh:mm:ss");
            other.State = ObjectState.Modified;

            //a new data access class (new DbContext)
            using (DataAccessor b = new DataAccessor())
            {
                //single method to handle inserts and updates
                b.InsertOrUpdate<Primary>(existing);
            }

            //return the values to the original ones
            existing.Description = existingValue;
            other.AlternateDescription = existingOtherValue;
            existing.State = ObjectState.Modified;
            other.State = ObjectState.Modified;

            using (DataAccessor c = new DataAccessor())
            {
                //update the entities back to normal
                c.InsertOrUpdate<Primary>(existing);
            }
        }

        [TestMethod]
        public void UpdateManyToManySingleContext()
        {
            Primary existing = null;
            Secondary other = null;
            String existingValue = String.Empty;
            String existingOtherValue = String.Empty;

            using (DataAccessor a = new DataAccessor())
            {
                existing = a.DataContext.Primaries.Include("Secondaries").First();
                Assert.IsTrue(existing.Secondaries.Count() > 1, "Should be at least 1 linked item");


                existingValue = existing.Description;
                existing.Description = "Edit " + DateTime.Now.ToString("yyyyMMdd hh:mm:ss");
                existing.State = ObjectState.Modified;

                other = existing.Secondaries.First();
                existingOtherValue = other.AlternateDescription;
                other.AlternateDescription = "Edit " + DateTime.Now.ToString("yyyyMMdd hh:mm:ss");
                other.State = ObjectState.Modified;

                a.InsertOrUpdate<Primary>(existing);

                existing.Description = existingValue;
                other.AlternateDescription = existingOtherValue;
                existing.State = ObjectState.Modified;
                other.State = ObjectState.Modified;

                a.InsertOrUpdate<Primary>(existing);
            }            
        }

        [TestMethod]
        public void InsertManyToManyThenDelete()
        {
            Primary existing = null;

            using (DataAccessor a = new DataAccessor())
            {
                existing = a.DataContext.Primaries.Include("Secondaries").First();
                Assert.IsTrue(existing.Secondaries.Count() > 1, "Should be at least 1 linked item");
            }

            Secondary s = new Secondary();
            s.SecondaryId = 100;
            s.AlternateTitle = "Unit Test";
            s.AlternateDescription = "Unit " + DateTime.Now.ToString("yyyyMMdd hh:mm:ss");
            s.State = ObjectState.Added;

            existing.Secondaries.Add(s);

            using (DataAccessor b = new DataAccessor())
            {
                b.InsertOrUpdate<Primary>(existing);
            }

            Other o = new Other();
            o.Name = "UT " + DateTime.Now.ToString("yyyyMMdd hh:mm:ss");
            o.OtherId = 1000;
            o.State = ObjectState.Added;
            
            s.Other = o;
            s.OtherId = o.OtherId;
            s.State = ObjectState.Modified;

            using (DataAccessor b2 = new DataAccessor())
            {
                b2.InsertOrUpdate<Primary>(existing);
            }

            using (DataAccessor c = new DataAccessor())
            {
                //c.RemoveEntities<Primary, Secondary>(existing, "Secondaries", s); (or can use an expression):
                c.RemoveEntities<Primary, Secondary>(existing, x => x.Secondaries, s);
                s.Other = null;
                s.OtherId = null;
                s.State = ObjectState.Modified;
                o.State = ObjectState.Unchanged;
                c.InsertOrUpdate<Secondary>(s);
            }

            using (DataAccessor d = new DataAccessor())
            {                
                d.Delete<Secondary>(s);
                d.Delete<Other>(o);
            }
        }

        [TestMethod]
        public void InsertManyToManyThenDeleteSingleContext()
        {
            Primary existing = null;

            using (DataAccessor a = new DataAccessor())
            {
                existing = a.DataContext.Primaries.Include("Secondaries").First();
                Assert.IsTrue(existing.Secondaries.Count() > 1, "Should be at least 1 linked item");

                Secondary s = new Secondary();
                s.SecondaryId = 100;
                s.AlternateTitle = "Unit Test";
                s.AlternateDescription = "Unit " + DateTime.Now.ToString("yyyyMMdd hh:mm:ss");
                s.State = ObjectState.Added;
                existing.Secondaries.Add(s);

                a.InsertOrUpdate<Primary>(existing);

                Other o = new Other();
                o.Name = "UT " + DateTime.Now.ToString("yyyyMMdd hh:mm:ss");
                o.OtherId = 1000;
                o.State = ObjectState.Added;

                s.Other = o;
                s.OtherId = o.OtherId;
                s.State = ObjectState.Modified;

                a.InsertOrUpdate<Primary>(existing);

                //c.RemoveEntities<Primary, Secondary>(existing, "Secondaries", s); (or can use an expression):
                a.RemoveEntities<Primary, Secondary>(existing, x => x.Secondaries, s);
                s.Other = null;
                s.OtherId = null;
                s.State = ObjectState.Modified;
                o.State = ObjectState.Unchanged;
                a.InsertOrUpdate<Secondary>(s);

                a.Delete<Secondary>(s);
                a.Delete<Other>(o);
            }            
        }
    }
}
