﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Data.Entity.Migrations;
using System.Diagnostics;
using Example.DataAccess.HelperClasses;
using System.Data.Entity.Infrastructure;
using System.Data.Entity.Core.Objects;
using System.Linq.Expressions;
using System.Data.Entity.Validation;

namespace Example.DataAccess
{
    public class DataAccessor : IDisposable
    {
        private SampleEntities _accessor;

        public SampleEntities DataContext { get { return _accessor; } }

        public DataAccessor()
        {
            _accessor = new SampleEntities();
            _accessor.Configuration.LazyLoadingEnabled = false;
            _accessor.Configuration.ProxyCreationEnabled = false;
        }

        /// <summary>
        /// Determines if an entity exists in the Data Context or in the physical data store
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="entity"></param>
        /// <returns>Returns null if not found, else returns the located entity which may not be the same as the entity passed in</returns>
        protected T Exists<T>(T entity) where T : EntityBase
        {
            var objContext = ((IObjectContextAdapter)this.DataContext).ObjectContext;
            var objSet = objContext.CreateObjectSet<T>();
            var entityKey = objContext.CreateEntityKey(objSet.EntitySet.Name, entity);

            DbSet<T> set = DataContext.Set<T>();
            var keys = (from x in entityKey.EntityKeyValues
                        select x.Value).ToArray();

            //Remember, there can by surrogate keys, so don't assume there's just one column/one value
            //If a surrogate key isn't ordered properly, the Set<T>().Find() method will fail, use attributes on the entity to determien the proper order.

            //context.Configuration.AutoDetectChangesEnabled = false;
            //http://stackoverflow.com/questions/11686225/dbset-find-method-ridiculously-slow-compared-to-singleordefault-on-id

            return set.Find(keys);
        }

        private void ApplyStateChanges<T>(params T[] items) where T : EntityBase
        {
            Debug.WriteLine("Started ApplyStateChanges");

            DbSet<T> dbSet = DataContext.Set<T>();
            foreach (T item in items)
            {
                Debug.WriteLine(String.Format("Item: {0}|{1}", item.State, item.GetType()));

                dbSet.Attach(item); //loads related entities into the current context

                if (item.State == ObjectState.Added || item.State == ObjectState.Modified)
                {
                    dbSet.AddOrUpdate(item);
                }
                else if (item.State == ObjectState.Deleted)
                {
                    dbSet.Remove(item);
                }

                foreach (DbEntityEntry<EntityBase> entry in DataContext.ChangeTracker.Entries<EntityBase>()
                    .Where(c => c.Entity.State != ObjectState.Processed && c.Entity.State != ObjectState.Unchanged))
                {
                    Debug.WriteLine(String.Format("Entry: {0}|{1}", entry.Entity.State, entry.Entity.GetType()));

                    var y = DataContext.Entry(entry.Entity);              
                    y.State = HelperFunctions.ConvertState(entry.Entity.State);
                    entry.Entity.State = ObjectState.Processed;
                }
            }
            Debug.WriteLine("Finished ApplyStateChanges");
        }
        
        public void InsertOrUpdate<T>(params T[] entities) where T : EntityBase
        {
            try
            {
                ApplyStateChanges(entities);
                DataContext.SaveChanges();
            }
            catch (Exception e)
            {
                PrintError(e);

                DbEntityValidationException ev = e as DbEntityValidationException;
                if (ev != null)
                {
                    foreach (var error in ev.EntityValidationErrors)
                    {
                        foreach(var em in error.ValidationErrors)
                        {
                            Debug.WriteLine(em.ErrorMessage);
                        }
                    }
                }                
            }
        }

        private void PrintError(Exception ex)
        {
            if (ex == null)
                return;

            Debug.WriteLine(ex.Message);

            if (ex.InnerException != null)
            {
                PrintError(ex.InnerException);
            }
        }

        public void RemoveEntities<T, T2>(T parent, string collectionName, params T2[] children) 
            where T : EntityBase 
            where T2 : EntityBase
        {         
            DataContext.Set<T>().Attach(parent);
            ObjectContext obj = DataContext.ToObjectContext();
            foreach(T2 child in children)
            {
                DataContext.Set<T2>().Attach(child);
                obj.ObjectStateManager.ChangeRelationshipState(parent, child, collectionName, EntityState.Deleted);
            }
            DataContext.SaveChanges();            
        }

        public void RemoveEntities<T, T2>(T parent, Expression<Func<T, object>> expression, params T2[] children)
            where T : EntityBase
            where T2 : EntityBase
        {
            DataContext.Set<T>().Attach(parent);
            ObjectContext obj = DataContext.ToObjectContext();
            foreach (T2 child in children)
            {
                DataContext.Set<T2>().Attach(child);
                obj.ObjectStateManager.ChangeRelationshipState(parent, child, expression, EntityState.Deleted);
            }
            DataContext.SaveChanges();         
        }

        /// <summary>
        /// Sets an entity to be removed on the next Save Changes call
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="entity"></param>
        /// <returns></returns>
        public void Delete<T>(params T[] entities) where T : EntityBase
        {
            foreach (T entity in entities)
            {
                T attachedEntity = Exists<T>(entity);

                if (attachedEntity != null)
                {
                    var attachedEntry = DataContext.Entry(attachedEntity);
                    attachedEntry.State = EntityState.Deleted;
                }
            }
            DataContext.SaveChanges();
        }

        public void Dispose()
        {
            if (_accessor != null)
            {
                _accessor.Dispose();
                _accessor = null;
            }
        }
    }
}
